/***************************************************************************
 *        GOnOffFitterSpectrum.hpp - ON-OFF spectral analysis class        *
 * ----------------------------------------------------------------------- *
 *  copyright (C) 2009-2013 by Juergen Knoedlseder                         *
 * ----------------------------------------------------------------------- *
 *                                                                         *
 *  This program is free software: you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation, either version 3 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 *                                                                         *
 ***************************************************************************/
/**
 * @file GOnOffFitterSpectrum.hpp
 * @brief ON-OFF spectral analysis class
 * @author Pierrick Martin
 */

#ifndef GONOFFFITTERSPECTRUM_HPP
#define GONOFFFITTERSPECTRUM_HPP

/* __ Includes ___________________________________________________________ */
#include <string>
#include <vector>
#include "GObservations.hpp"
#include "GEbounds.hpp"
#include "GEnergy.hpp"
#include "GOnOffMaker.hpp"
#include "GModelSpectral.hpp"


/***********************************************************************//**
* @class GOnOffFitterSpectrum
*
* @brief ON-OFF spectral analysis class
*
* This class implements the spectral analysis of data with an ON-OFF
* approach for the evaluation of the instrumental background.
*
* Two kinds of analyses can be considered: either the global fit of 
* given spectral shape over the entire energy grid, or the determination
* of flux points for all energy bins. The selection of one or the other
* option is done from whether or not a GModelSpectral object is passed on
* instantiation. In the case of flux points computation, an instance of 
* GModelSpectralFunc is created internally and filled with the results.
*
* The method convolve_spectrum() performs the convolution of the spectral 
* model with the instrument response and fills the m_expcts variable.
* Note that the convolution is done for each observation run and we store
* here only the summed counts.
*
* The method compute_energy() computes the average true energy and its 
* variance for each reconstructed energy bin and fills the m_truemean 
* and m_truevar variables. Both cannot be accessed at the moment.
* 
* The method compute_significance() evaluates the significance of the 
* source signal in each energy bin from the Li&Ma-1983 prescription.
* The result can be accessed through the significance() method.
*
* The method sum_data() sets the m_sum flag to indicate if the analysis
* is performed on summed ON and OFF data or keeping the ON and OFF
* information of each run independent. This implies a separate 
* optimization function for each case.
*
* The results, either best-fit spectral parameters or best-fit flux points,
* are written to an XML file through the methods of the GModelSpectral 
* object.
*
* To be decided:
* - Output format for the significance ? Just in log file ?
* - Do we store expected/observed counts per run in GOnOffBin ?
* - Do we want npred/eval/gradient methods as in GModel 
*   (working on GonOffBin instead of GEvent) ?
*
***************************************************************************/
class GOnOffFitterSpectrum : public GOnOffFitter {
	
public:
    // Constructors and destructors
    GOnOffFitterSpectrum(void);
    GOnOffFitterSpectrum(const GOnOffFitterSpectrum& onoff);
	explicit GOnOffFitterSpectrum(const std::string& filename, const GOnOffMaker& maker);
	explicit GOnOffFitterSpectrum(const GObservations& obs, const GOnOffMaker& maker);
	explicit GOnOffFitterSpectrum(const std::string& filename, const GOnOffMaker& maker, const GModelSpectral& spectral);
	explicit GOnOffFitterSpectrum(const GObservations& obs, const GOnOffMaker& maker, const GModelSpectral& spectral);
	explicit GOnOffFitterSpectrum(const std::string& filename, const GOnOffMaker& maker, const std::string& spec);
	explicit GOnOffFitterSpectrum(const GObservations& obs, const GOnOffMaker& maker, const std::string& spec);
    virtual ~GOnOffFitterSpectrum(void);
	
    // Operators
    virtual GOnOffFitterSpectrum&    operator=(const GOnOffFitterSpectrum& onoff);
	
    // Implemented pure virtual base class methods
    virtual void                     clear(void);
    virtual GOnOffFitterSpectrum*    clone(void) const;
    virtual int                      size(void) const;
	virtual std::string              type(void) const;
    virtual std::string              print(const GChatter& chatter = NORMAL) const;
	virtual void                     optimize(GOptimizer& opt);
	
	// Methods specific to the derived class
	void                             spectral(const GModelSpectral& spectral);
    void                             spectral(const std::string& filename);
    const GModelSpectral*            spectral(void) const;
	void                             sum_data(void);
	const std::vector<double>        significance(void) const;
	const std::vector<GEnergy>       true_energy(void) const;
	const std::vector<GEnergy>       true_variance(void) const;
    void                             convolve_spectrum(); 
	void                             compute_energy();
	void                             compute_significance();
	
protected:
    // Protected methods
    void init_members(void);
    void copy_members(const GOnOffFitterSpectrum& obs);
    void free_members(void);
	
    // Protected members
	GModelSpectral*              m_spectral;   //!< Spectral model
	std::vector<double>          m_expcts;     //!< Array of expected counts in all reconstructed energy bins (summed over all runs)
	std::vector<double>          m_obscts;     //!< Array of observed counts in all reconstructed energy bins (summed over all runs)	
    std::vector<GEnergy>         m_truemean;   //!< Array of average true energy for all reconstructed energy bins
    std::vector<GEnergy>         m_truevar;    //!< Array of true energy variance for all reconstructed energy bins
	std::vector<double>          m_sig;        //!< Array of significances of each flux point
	bool                         m_sum         //!< Flag to tell if analysis to be done on summed ON/OFF data
};

#endif /* GONOFFFITTERSPECTRUM_HPP */
